@@ -1,16 +1,32 @@
 # -*- coding: utf-8 -*-
 """
-to convert nmea files to snr files 
-Makan Karegar, Uni Bonn, Aug 05, 2021
+convert nmea files to snr files 
 """
 from __future__ import division
 import numpy as np 
 import os
 import subprocess
 from scipy.interpolate import interp1d
+
 import gnssrefl.gps as g
 
 def NMEA2SNR(locdir, fname, snrfile, csnr):
+    """
+
+    Parameters
+    -----------
+    locdir : str
+        directory where your SNR files are kept
+    fname : string
+        NMEA filename 
+
+    snrfile : str
+        name of output file for SNR data
+
+    csnr : str
+        snr option, i.e. '66' or '99'
+
+    """
     
     missing = True
 
@@ -79,8 +95,29 @@ def NMEA2SNR(locdir, fname, snrfile, csnr):
 def read_nmea(fname):
     """
     read GPGGA sentence (includes snr data) in NMEA files    
-    Makan Karegar, Uni Bonn    
-    Dec, 2019
+
+    Parameters
+    ----------
+    fname : string
+        NMEA filename
+
+    Returns
+    -------
+    t : list of integers
+        timetags in GPS seconds 
+
+    prn : list of integers
+        GPS satellite numbers
+
+    az : list of floats ??
+        azimuth values (degrees)
+
+    elv : list of floats ??
+        elevation angles (degrees)
+
+    snr : list of floats
+        snr values
+
     """
     
     f=open(fname, 'rb')
@@ -158,9 +195,27 @@ def read_nmea(fname):
 
 def fix_angle_azimuth(time, angle, azimuth):
     """
-    interpolare elevation angles and azimuth to retrieve decimal values thru time     
-    Makan Karegar, Uni Bonn    
-    Feb, 2020
+    interpolate elevation angles and azimuth to retrieve decimal values thru time     
+    this is for NMEA files.
+
+    Parameters
+    ----------
+    time : list of floats
+        GPS seconds of the week
+    angle : list of floats
+        elevation angles  (degrees)
+
+    azimuth : list of floats
+        azimuth angles (degrees)
+
+    Returns
+    -------
+    angle_fixed : list of floats
+        interpolated elevation angles
+
+    azim_fixed : list of floats
+        interpolated azimuth angles
+
     """
     
 #delet nans
@@ -206,6 +261,12 @@ def azimuth_diff2(azim1, azim2):
     return diff
 
 def azimuth_diff1 (azim):
+    """
+    Parameters
+    ----------
+    azim: ??
+
+    """
     azim_a = azim[0:-1]
     azim_b = azim[1:]
     diff = azimuth_diff2 (azim_b, azim_a);
@@ -213,6 +274,14 @@ def azimuth_diff1 (azim):
     return diff
 
 def azimuth_diff(azim1, azim2):
+    """
+    Parameters
+    ----------
+    azim1 : ??
+
+    azim2 : ??
+
+    """
     if not(azim2.size):
         diff = azimuth_diff1 (azim1)
     else:
@@ -221,6 +290,13 @@ def azimuth_diff(azim1, azim2):
     return diff
      
 def angle_range_positive(ang):
+    """
+
+    Parameters
+    ----------
+    ang : ??
+
+    """
 #    idx1 = np.isfinite(ang)
     ang = np.angle(np.exp(1j*ang*np.pi/180))*180/np.pi  
     idx2 = (ang < 0)
@@ -228,6 +304,20 @@ def angle_range_positive(ang):
     return ang
 
 def azimuth_mean(azim1, azim2):
+    """
+    Parameters
+    ----------
+    azim1 : list of floats ? 
+         azimuth degrees
+
+    azim2 : list of floats
+         azimuth degrees
+
+    Returns
+    ---------
+    azim : float 
+        degrees
+    """
     azim = np.concatenate([azim1, azim2])
     if np.all(azim1 >= 0) and np.all(azim2 >= 0):
         azim1 = angle_range_positive(azim1)
@@ -238,13 +328,28 @@ def azimuth_mean(azim1, azim2):
         y1 = np.cos(azim1*np.pi/180);y2 = np.cos(azim2*np.pi/180) 
         x = ( (x1 + x2)/2.0 )[0];y = ( (y1 + y2)/2.0 )[0]
         azim = 180/np.pi * np.arctan2(x, y)
+
     return azim
 
 def quickname(station,year,cyy, cdoy, csnr):
     """
     given station name, year, doy, snr type
     returns snr file name and its path under REFL_CODE/year/snr/station/
-    author: Makan Karegar
+
+    Parameters
+    ----------
+    station : str
+
+    year : int
+
+    cyy : str 
+        two character yar
+
+    cdoy : str
+        three character day of year
+
+    csnr : str
+        snr type, e.g. '66' 
     """
     
     xdir  = os.environ['REFL_CODE'] + '/'
@@ -255,10 +360,21 @@ def quickname(station,year,cyy, cdoy, csnr):
 
 def elev_limits(snroption):
     """
-    given Kristine's snr option, return min and max elevation angles
-    in degrees
-    author: kristine larson
-    2020 august 7
+
+    given snr option return the elevation angle limits
+
+    Parameters
+    ----------
+    snroption : int
+        snrfile number
+
+    Returns
+    -------
+    emin : float
+        min elevation angle (degrees)
+    emax : float
+        max elevation angle (degrees)
+
     """
 
     if (snroption == 99):
@@ -276,14 +392,25 @@ def elev_limits(snroption):
   
 def run_nmea2snr(station, year_list, doy_list, isnr, overwrite):
     """
-    runs the nmea2snr conversion
-    inputs:
-    station name 
-    year_list list of years to be analyzed
-    doy_list list of doy to be analyzed
-    isnr = integer file type choice
-    overwrite = boolean, make a new SNR file even if one already exists
-    Makan Karegar
+    runs the nmea2snr conversion code
+
+    Parameters
+    ----------
+    station : str
+        name of station 
+
+    year_list : list of integers
+        years 
+
+    doy_list : list of days of year
+        days of years
+
+    isnr : int
+        snr file type
+
+    overwrite : bool
+        whether make a new SNR file even if one already exists
+
     """
     # loop over years and day of years
     for yr in year_list:
