@@ -10,12 +10,15 @@ import subprocess
 import sys
 import time
 
+
 # progress bar for RINEX translation/orbits
 from progress.bar import Bar
 
 # my gps libraries
 import gnssrefl.gps as g
 import gnssrefl.rinpy as rinpy
+import gnssrefl.karnak_libraries as k
+import gnssrefl.cddis_highrate as ch
 
 # fortran codes for translating RINEX
 import gnssrefl.gpssnr as gpssnr
@@ -31,38 +34,98 @@ class constants:
 #
 #
 def quickname(station,year,cyy, cdoy, csnr):
+    """
+    finds location/name of a SNR file
+
+    Parameters 
+    ----------
+    station : str
+        station name, 4 character
+
+    year : int
+        full year
+
+    cyy : str 
+        two character year
+
+    cdoy : str
+        three character day of year
+
+    csnr : str
+        snr ending
+
+    Returns
+    -------
+    fname : str
+        filename
+
+    """
     xdir  = os.environ['REFL_CODE'] + '/'
     fname =  xdir + str(year) + '/snr/' + station + '/' + station + cdoy + '0.' + cyy + '.snr' + csnr
     return fname
 
-def run_rinex2snr(station, year_list, doy_list, isnr, orbtype, rate,dec_rate,archive,fortran,nol,overwrite,translator,srate,mk,skipit):
+def run_rinex2snr(station, year_list, doy_list, isnr, orbtype, rate,dec_rate,archive,fortran,nol,overwrite,translator,srate,
+        mk,skipit,stream,strip):
     """
-    runs the rinex 2 snr conversion
-    inputs:
-    station name 
+    main code to convert RINEX files into SNR
+
+    Parameters
+    ----------
+    station: string
+        4 or 9 character station name 
+
     year_list list of years to be analyzed
+
     doy_list list of doy to be analyzed
-    isnr = integer file type choice
-    orbtype = 3character orbit type
-    rate = high or low
-    dec_rate = integer for decimation
-    archive = allows you to pick a specific archive
-    fortran = boolean, whether you use fortran rinex translators
-    nol = boolean for nolook, if set to True, then it will assume RINEX files are in local directory
-    overwrite = boolean, make a new SNR file even if one already exists
-    translate = string with .... fortran, python, hybrid ??
-    srate = sample rate
-    mk = makan option
-    skipit = skips making files every day, so a value of 7 means weekly.  1 means do every day
+
+    isnr: integer 
+        SNR file type choice
+
+    orbtype: string
+        3 character orbit type, e.g. nav
+        
+    rate : string 
+        general sample rate. 
+        high: use 1-Hz area in the archive
+        low: use default area in the archive
+
+    dec_rate : integer
+         decimation value
+
+    archive : string
+        choice of GNSS archive
+
+    fortran : boolean 
+        whether the fortran rinex translator is to be used
+        default: false
+
+    nol: boolean 
+        True: assumes RINEX files are in local directory
+        False (default): will look at multiple - or specific archive
+
+    overwrite: boolean
+        False (default): if SNR file exists, SNR file not made
+        True: make a new SNR file 
+
+    translator : string 
+        hybrid (default), fortran, or python
+        hybrid uses fortran within the python code
+
+    srate: integer
+        sample rate for RINEX 3 files
+
+    mk : boolean
+        makan option
+
+    skipit : boolean
+         skips making files every day, so a value of 7 means weekly.  1 means do every day
+
+    strip : boolean
+         reduces observables to only SNR (too many observables, particularly in RINEX 2 files
+         will break the RINEX translator)
 
-    2021feb11, kristine Larson
-    translator = fortran, python, or hybrid
-    srate - integer sample rate, for RINEX 3 only.
-    2021mar20 check for illegal day of years ....
-    2021aug01 added mk option for uppercase file names per makan karegar request
-    2021aug15 added weekly option
-    2021nov08 making nolook work for rinex 3 files ...
     """
+    # 
     # do not allow illegal skipit values
     if skipit < 1:
         skipit = 1
@@ -84,18 +147,13 @@ def run_rinex2snr(station, year_list, doy_list, isnr, orbtype, rate,dec_rate,arc
         if not mk:
             station = station[0:4].lower()
     else:
-        print('Illegal station input - Station must have 4 or 9 characters. Exiting')
+        print('Illegal station input - Station must have 4,6,or 9 characters. Exiting')
         sys.exit()
     year_st = year_list[0]
     year_end = year_list[-1]
     doy_st = doy_list[0]
     doy_end = doy_list[-1]
-    # 2021 september 12 KL
-    # i would like to put this in rinex2snr_cl.py - but I am trying to avoid increasing
-    # the workload for the Notebook programmers before the short course.
-    #print(year_st,year_end,doy_st, doy_end)
 
-# this way we are overwriting the doy_list variable
 # loop thru years and days 
     for year in year_list:
         ann = g.make_nav_dirs(year)
@@ -116,18 +174,19 @@ def run_rinex2snr(station, year_list, doy_list, isnr, orbtype, rate,dec_rate,arc
             cdoy = '{:03d}'.format(doy) ; cyy = '{:02d}'.format(year-2000)
             # first, check to see if the SNR file exists
             fname =  quickname(station,year,cyy,cdoy,csnr)
+            # now it unzips if that version exists
             snre = g.snr_exist(station,year,doy,csnr)
             if snre:
-                print('SNR file exists', fname)
                 if overwrite:
-                    #print('you requested it be overwritten, so removing file')
-                    subprocess.call(['rm', fname])
-                    snre = False
+                    print('File exists, you requested overwriting, so will delete existing file')
+                    subprocess.call(['rm', fname]); snre = False
+                else:
+                    print('SNR file already exists', fname)
+
             illegal_day = False
             if (doy > dec31):
                 illegal_day = True
-                #print('illegal day',illegal_day, doy, dec31)
-            # combining these so i don't have to indent everything
+
             if (not illegal_day) and (not snre):
                 r = station + cdoy + '0.' + cyy + 'o'
                 rgz = station + cdoy + '0.' + cyy + 'o.gz'
@@ -137,24 +196,31 @@ def run_rinex2snr(station, year_list, doy_list, isnr, orbtype, rate,dec_rate,arc
                     if version == 2:
                         the_makan_option(station,cyyyy,cyy,cdoy) # looks everywhere in your local directories
                         if os.path.exists(r):
-                            #print('RINEX 2 file exists locally')
+                            if strip:
+                                print('Testing out stripping the RINEX 2 file here')
+                                k.strip_rinexfile(r)
                             conv2snr(year, doy, station, isnr, orbtype,rate,dec_rate,archive,fortran,translator) 
                         else:
                             print('You Chose the No Look Option, but did not provide the needed RINEX file.')
                     if version == 3:
-##                        print(' 2021feb22 using sample rate explicitly')
-                        # make sure sample rate takes up two characters ...
-                        csrate = '{:02d}'.format(srate)
-                        r3 = station9ch + '_R_' + str(year) + cdoy + '0000_01D_' + csrate + 'S_MO.rnx'
-                        r3gz = station9ch + '_R_' + str(year) + cdoy + '0000_01D_' + csrate + 'S_MO.rnx.gz'
+                        if rate == 'high':
+                            csrate = '01' # high rate assumes 1-sec
+                        else:
+                            csrate = '{:02d}'.format(srate)
+                        streamid = '_' + stream  + '_'
+                        # this can be done in a function now ... 
+                        r3cmpgz = station9ch + streamid + str(year) + cdoy + '0000_01D_' + csrate + 'S_MO.crx.gz'
+                        r3 = station9ch + streamid + str(year) + cdoy + '0000_01D_' + csrate + 'S_MO.rnx'
+                        r3gz = station9ch + streamid + str(year) + cdoy + '0000_01D_' + csrate + 'S_MO.rnx.gz'
                         r2 = station + cdoy + '0.' + cyy + 'o'
+                        if os.path.exists(r3cmpgz):
+                            translated, rnx_filename = go_from_crxgz_to_rnx(r3cmpgz)
                         if os.path.exists(r3gz):
                             subprocess.call(['gunzip', r3gz])
                         if os.path.exists(r3):
                             print('The RINEX 3 file exists locally')
-                            fexists = g.new_rinex3_rinex2(r3,r2)
+                            fexists = g.new_rinex3_rinex2(r3,r2,dec_rate)
                             if fexists:
-                                #print('Rinex 3 to 2 conversion worked, now convert to snr format')
                                 conv2snr(year, doy, station, isnr, orbtype,rate,dec_rate,archive,fortran,translator) 
                             else:
                                 print('Something about the RINEX 3-2 conversion did not work')
@@ -163,53 +229,95 @@ def run_rinex2snr(station, year_list, doy_list, isnr, orbtype, rate,dec_rate,arc
                             print('I assumed its name was ', r3)
 
                 else:
-                    print('Will seek the RINEX file externally for ', station, ' year:', year, ' doy:', doy, ' translate with ', translator, ' at the ', archive, ' archive')
+                    print('Will seek the RINEX file from an external archive')
                     if version == 3:
-                        #  try unavco
+                        fexists = False
+                        rnx_filename = '' # just in  case?
+                        print(station9ch, ' year:', year, ' doy:', doy, 'from: ', archive)
+                        r2 = station + cdoy + '0.' + cyy + 'o'
                         rinex2exists = False; rinex3name = '';
-                        if (archive == 'unavco') or (archive == 'all'):
-                            srate = 15
-                            rinex2exists, rinex3name = g.unavco_rinex3(station9ch, year, doy,srate,orbtype)
+                        if (rate == 'high'):
+                            print('This code only accesses 1-Hz Rinex 3 data at CDDIS, BKG, and GA')
+                            if archive == 'ga':
+                                deleteOld = True
+                                # cold should return the new name of the rinex 2 file
+                                r2, foundit = g.ga_highrate(station9ch,year,doy,dec_rate,deleteOld)
+                                if foundit: 
+                                    print('rinex2 file should now exist:', r2)
+                            if archive == 'cddis':
+                                rnx_filename,foundit = ch.cddis_highrate(station9ch, year, doy, 0,stream,dec_rate)
+                                if foundit:
+                                    print('The RINEX 3 file has been downloaded. Try to make ', r2)
+                                    fexists = g.new_rinex3_rinex2(rnx_filename,r2,dec_rate)
+                            if archive == 'bkg':
+                                rnx_filename,foundit = ch.bkg_highrate(station9ch, year, doy, 0,stream,dec_rate)
+                                if foundit:
+                                    print('The RINEX 3 file has been downloaded and merged. Try to make ', r2)
+                                    fexists = g.new_rinex3_rinex2(rnx_filename,r2,dec_rate)
 
-                        # try cddis
-                        if not os.path.exists(r):
-                            if (archive == 'cddis') or (archive == 'all'):
-                                srate = 30
-                                rinex2exists, rinex3name = g.cddis_rinex3(station9ch, year, doy,srate,orbtype)
-                        if not os.path.exists(r):
-                            if (archive == 'ga') :
-                                srate = 30
-                                rinex2exists, rinex3name = g.ga_rinex3_to_rinex2(station9ch, year, doy,srate)
-
-                        # remove rinex 3 file
-                        # subprocess.call(['rm', '-f', rinex3name]) # remove rinex3 file
-
-                        if rinex2exists:
-                            print('RINEX version 2 has been created from version 3', year, doy)
-                            conv2snr(year, doy, station, isnr, orbtype,rate,dec_rate,archive,fortran,translator) 
                         else:
-                            print('RINEX file does not exist for ', year, doy)
+                            if (archive == 'all'):
+                                file_name,foundit = k.universal_all(station9ch, year, doy,srate,stream)
+                                if (not foundit): # try again
+                                    file_name,foundit = k.universal_all(station9ch, year, doy, srate,k.swapRS(stream))
+                            else:
+                                #print('stream',stream)
+                                file_name,foundit = k.universal(station9ch, year, doy, archive,srate,stream)
+                                if (not foundit): # try again
+                                    #print('stream',stream)
+                                    file_name,foundit = k.universal(station9ch, year, doy, archive,srate,k.swapRS(stream))
+                            if foundit: # version 3 found - now need to gzip, then hatanaka decompress
+                                translated, rnx_filename = go_from_crxgz_to_rnx(file_name)
+                            # now make rinex2
+                                if translated:
+                                    print('The RINEX 3 file has been downloaded. Try to make ', r2)
+                                    fexists = g.new_rinex3_rinex2(rnx_filename,r2,dec_rate)
+                                    #subprocess.call(['rm', '-f',rnx_filename]) # rnx
+                        # this means the rinex 2 version exists
+                        if fexists:
+                             print('RINEX 2 created from v3', year, doy, ' Now remove RINEX 3 files and convert')
+                             subprocess.call(['rm', '-f',rnx_filename]) # rnx
+                             conv2snr(year, doy, station, isnr, orbtype,rate,dec_rate,archive,fortran,translator) 
+                        else:
+                            print('Unsuccessful RINEX 3 retrieval/translation', year, doy)
                     else:
-                        # this is rinex version 2
+                        print(station, ' year:', year, ' doy:', doy, 'from: ', archive)
+                        # this is rinex version 2 - finds rinex and converts it
                         conv2snr(year, doy, station, isnr, orbtype,rate,dec_rate,archive,fortran,translator) 
 
-    #print('And I guess my work is done now!')
 
 def conv2snr(year, doy, station, option, orbtype,receiverrate,dec_rate,archive,fortran,translator):
     """
-    inputs: year and day of year (integers) and station name
-    option is for the snr creation ??? integer or character?
-    orbtype can be nav or sp3.  if the former, then gpsSNR is used.
-    if the later, then gnssSNR
-    what are receiverrate and dec_rate defaults?
-    this assumes you follow my definitions for where things go,
-    i.e. REFL_CODE and ORBITS
-    it currently checks Unavco, SOPAC, and SONEL. I should add CDDIS
-    author: kristine m. larson
-    19may20, added decimation
-    19sep12, I got tired of code crashing for files > 20 observables.  I am thus using teqc
-    20apr15, xz compression added but also try to streamline it.
-    20jul10, added arvchive setting. default is 'all'
+    convert RINEX files to SNR files
+
+    Parameters
+    ----------
+    year : int
+        full year
+
+    doy : int
+        day of year
+
+    option : int
+        snr choice (66, 99 etc)
+
+    orbtype : str
+        orbit source (nav, gps, gnss, etc)
+
+    receiverrate : int
+        sampling interval of the GPS receiver, e.g. 1, 30, 15
+
+    dec_rate : int
+        decimation value to reduce file size
+
+    archive : str
+        location of the rinex files
+
+    fortran : bool
+         whether fortran translator to be used.  this is here for backwards compatability
+
+    translator : str
+         hybrid, python, or fortran 
 
     """
     # define directory for the conversion executables
@@ -232,21 +340,36 @@ def conv2snr(year, doy, station, option, orbtype,receiverrate,dec_rate,archive,f
         month = d.month; day = d.day
         # new function to do the whole orbit thing
         foundit, f, orbdir, snrexe = g.get_orbits_setexe(year,month,day,orbtype,fortran) 
-        # if you have the orbit file, you can get the rinex file
+        # if you have the orbit file, you can get the rinex file. First lets define the expected names
         print('Orbit file: ', orbdir + '/' + f)
         if foundit:
             # now you can look for a rinex file
             rinexfile,rinexfiled = g.rinex_name(station, year, month, day)
             # This goes to find the rinex file. I am changing it to allow 
             # an archive preference 
-            g.go_get_rinex_flex(station,year,month,day,receiverrate,archive)
+            if receiverrate == 'high':
+                strip_snr = False # for now - 
+                file_name, foundit = k.rinex2_highrate(station, year, doy,archive,strip_snr)
+            else:
+                # added karnak librariies
+                if (archive == 'all'):
+                    foundrinex = False
+                    for archivechoice in ['unavco','sopac','sonel']:
+                        if (not foundrinex):
+                            file_name,foundrinex = k.universal_rinex2(station, year, doy, archivechoice)
+                else:
+                    file_name,foundrinex = k.universal_rinex2(station, year, doy, archive)
+
+                if foundrinex: #uncompress etc  to make o files ...
+                    rinexfile, foundit2 = k.make_rinex2_ofiles(file_name) # translate
+
 #           define booleans for various files
             oexist = os.path.isfile(orbdir + '/' + f) == True
             rexist = os.path.isfile(rinexfile) == True
             exc = exedir + '/teqc' 
             texist = os.path.isfile(exc) == True
             if rexist: 
-                # decimate using teqc 
+                # decimate using teqc  if you have it
                 if (texist) and (fortran) and (dec_rate > 0): 
                     log.write("Decimating using teqc:  {0:3.0f}  seconds \n".format(dec_rate))
                     log.write('Unfortunately teqc removes Beidou data. Eventually I will remove this. \n')
@@ -280,7 +403,7 @@ def conv2snr(year, doy, station, option, orbtype,receiverrate,dec_rate,archive,f
                     in6 = g.binary(errorlog)
                     log.write('SNR file {0:50s} \n will use hybrid of python and fortran to make \n'.format( snrname))
                     # these are calls to the fortran codes that have been ported to be called from python
-                    if (orbtype  == 'gps') or (orbtype == 'nav'):
+                    if (orbtype  == 'gps') or ('nav' in orbtype):
                         gpssnr.foo(in1,in2,in3,in4,in5,in6)
                     else:
                         if (orbtype == 'ultra') or (orbtype == 'wum'):
@@ -342,12 +465,24 @@ def conv2snr(year, doy, station, option, orbtype,receiverrate,dec_rate,archive,f
 
 def satorb(week, sec_of_week, ephem):
     """
-    inputs are GPS week, seconds of the week, and the appropriate
-    ephemeris block from the navigation message
-    returns the x,y,z, coordinates of the satellite
-    and relativity correction (also in meters), so you add,
-    not subtract
-    Kristine Larson, April 2017
+    Calculate satellite orbits
+
+    Parameters
+    ----------
+
+    week : integer
+        GPS week
+
+    sec_of_week : float
+        GPS seconds of the week
+
+    ephem : ephemeris block
+
+    Returns 
+    -----------
+    numpy array 
+         the x,y,z, coordinates of the satellite in meters
+         and relativity correction (also in meters), so you add, not subtract
 
     """
 
@@ -394,18 +529,30 @@ def satorb(week, sec_of_week, ephem):
 
 def rnx2snr(obsfile, navfile,snrfile,snroption,year,month,day,dec_rate,log):
     """
-    inputs are obsfile - RINEX 2.11
-    navfile, which can be nav message or sp3 file
-    snrfile: where the results go
+    Converts a rinex v2.11 obs file using Joakim's rinex reading code
+
+    Parameters
+    ----------
+    obsfile : str
+        RINEX 2.11 filename
+    navfile : str
+        navigation file
+
+    snrfile : str 
+        SNR filename 
+
     snroption: integer
-    year,month,day:  what it sounds like, integer
-    dec_rate: integer I think, decimation rate. For fortran users, this is done with teqc
+        kind of SNR file requested
 
-    no output - the output is the snrfile that is created
+    year : int
+
+    month : int
+
+    day : int
+
+    dec_rate : integer 
+        decimation rate in seconds
 
-    This only works for rinex v2.11.  
-    Kristine M. Larson August 2020
-    This relies on Joakim's rinex reading code
     """
     station = obsfile[0:4]
     #logname = 'logs/' + station + 'python.txt'
@@ -470,6 +617,31 @@ def rnx2snr(obsfile, navfile,snrfile,snroption,year,month,day,dec_rate,log):
 
 def navorbits(navfile,obstimes,observationdata,obslist,prntoidx,gpssatlist,snrfile,s1exist,s2exist,s5exist,up,East,North,emin,emax,recv,dec_rate,log):
     """
+    Strandberg nav reading file?
+
+    Parameters 
+    ---------
+    navfile : string
+
+    obstimes : ??
+
+    observationdata :
+
+    obslist : 
+
+    prn2oidx : 
+
+    gpssatlist :
+
+    snrfile : str 
+        name of the output file
+
+    s1exist :
+
+    s2exist : 
+
+    s5exist :
+
     This is for GPS only files !
     navfile is nav broadcast ephemeris in RINEX format
     inputs are rinex info, obstimes, observationdata,prntoidx,gpssatlist
@@ -523,10 +695,23 @@ def navorbits(navfile,obstimes,observationdata,obslist,prntoidx,gpssatlist,snrfi
 
 def readSNRval(s1exist,s2exist,s5exist,observationdata,prntoidx,sat,i):
     """
-    what it looks like
-    only reads GPS data for now
+    what it looks like only reads GPS data for now
     interface between Joakim's code and mine ...
-    kristine m. larson, july 2019
+
+    Parameters 
+    ----------
+    s1exist : boolean
+
+    s2exist : boolean
+
+    s5exist : boolean
+
+    Returns 
+    -------
+    s1 : 
+    s2 :
+    s5 :
+
     """
     s1 = 0; s2 = 0; s5 = 0 # set defaults
     if s1exist:
@@ -551,7 +736,26 @@ def satorb_prop(week, secweek, prn, rrec0, closest_ephem):
     this assumes someone was nice enough to send you the closest ephemeris
     returns the satellite coordinates as well, so you can use htem
     in the A matrix
-    Kristine Larson, April 2017
+
+    Parameters
+    ----------
+    week : integer
+        GPS week
+
+    secweek : integer
+        GPS second of the week
+
+    prn : integer
+        satellite number
+
+    rrec0 : 3vector
+        receiver coordinates, meters
+
+    Returns
+    -------
+    SatOrbn : 3vector 
+        floats, Cartesian location of satellite in meters [x,y,z]
+
     """
     error = 1
 
@@ -586,6 +790,19 @@ def satorb_prop_sp3(iX,iY,iZ,recv,Tp,ij):
     and receiver coordinates rrec0
     find the x,y,z coordinates at time secweek
 
+    Parameters 
+    ----------
+    iX : float
+
+    iY : float
+
+    iZ : float
+    recv : 3 vector, float
+
+    Tp : 
+
+    ij : 
+
     sp3 has the orbit information in it
     """
     # start wit 70 milliseconds as the guess for the transmission time
@@ -727,10 +944,20 @@ def extract_snr(prn, con, obslist,obsdata,prntoidx,not_ij,emp):
 
 def elev_limits(snroption):
     """
-    given Kristine's snr option, return min and max elevation angles
-    in degrees
-    author: kristine larson
-    2020 august 7
+    For given SNR option, returns elevation angle limits
+
+    Parameters
+    ------------
+    snroption : integer
+        snr file delimeter
+
+    Returns
+    ----------
+    emin: float
+        minimum elevation angle (degrees)
+    emax: float
+        maximum elevation angle (degrees)
+
     """
 
     if (snroption == 99):
@@ -858,6 +1085,8 @@ def the_makan_option(station,cyyyy,cyy,cdoy):
     crnxpath = g.hatanaka_version()  # where hatanaka will be
     r = station + cdoy + '0.' + cyy + 'o'
     rd = station + cdoy + '0.' + cyy + 'd'
+    print(r,rd)
+
     locdir= os.environ['REFL_CODE'] + '/rinex/' + station + '/' + cyyyy + '/'
     # 
     #locdir2= os.environ['RINEX'] + station + '/' + cyyyy + '/'
@@ -947,4 +1176,33 @@ def the_makan_option(station,cyyyy,cyy,cdoy):
         else:
             g.hatanaka_warning()
 
+def go_from_crxgz_to_rnx(c3gz):
+    """
+    sent a gzipped hatanaka file.  gunzip, hatanaka translate
+    checks to see if the rnx version exists and returns that
+    and the filename
+    """
+    translated = False # assume failure
+    c3 = c3gz[:-3] # crx filename
+    rnx = c3.replace('crx','rnx') # rnx filename
+    # gunzip
+    if os.path.exists(c3gz):
+        subprocess.call(['gunzip', c3gz])
+
+    # executable
+    crnxpath = g.hatanaka_version()
+    if not os.path.exists(crnxpath):
+        g.hatanaka_warning()
+    else:
+        if os.path.exists(c3): # file exists
+            subprocess.call([crnxpath,c3])
+    if os.path.exists(rnx): # file exists
+        translated = True
+        #print('remove Hatanaka compressed file')
+        subprocess.call(['rm','-f',c3])
+
+    return translated, rnx
+
+
+
 
